@@ -25,8 +25,6 @@ use std::rc::Rc;
 use imgref::ImgVec;
 use rgb::RGBA8;
 
-use fnv::FnvHashMap;
-
 mod utils;
 
 mod text;
@@ -44,10 +42,8 @@ pub use text::{
 };
 
 use text::{
-    FontTexture,
+    GlyphAtlas,
     RenderMode,
-    RenderedGlyph,
-    RenderedGlyphId,
     TextContextImpl,
 };
 
@@ -86,7 +82,10 @@ use geometry::*;
 
 mod paint;
 pub use paint::Paint;
-use paint::PaintFlavor;
+use paint::{
+    GlyphTexture,
+    PaintFlavor,
+};
 
 mod path;
 use path::Convexity;
@@ -303,8 +302,9 @@ pub struct Canvas<T: Renderer> {
     height: u32,
     renderer: T,
     text_context: Rc<RefCell<TextContextImpl>>,
-    rendered_glyphs: FnvHashMap<RenderedGlyphId, RenderedGlyph>,
-    glyph_textures: Vec<FontTexture>,
+    glyph_atlas: Rc<GlyphAtlas>,
+    // Glyph atlas used for direct rendering of color glyphs, dropped after flush()
+    emphemeral_glyph_atlas: Option<Rc<GlyphAtlas>>,
     current_render_target: RenderTarget,
     state_stack: Vec<State>,
     commands: Vec<Command>,
@@ -328,8 +328,8 @@ where
             height: 0,
             renderer: renderer,
             text_context: Default::default(),
-            rendered_glyphs: Default::default(),
-            glyph_textures: Default::default(),
+            glyph_atlas: Default::default(),
+            emphemeral_glyph_atlas: Default::default(),
             current_render_target: RenderTarget::Screen,
             state_stack: Default::default(),
             commands: Default::default(),
@@ -356,8 +356,8 @@ where
             height: 0,
             renderer: renderer,
             text_context: text_context.0,
-            rendered_glyphs: Default::default(),
-            glyph_textures: Default::default(),
+            glyph_atlas: Default::default(),
+            emphemeral_glyph_atlas: Default::default(),
             current_render_target: RenderTarget::Screen,
             state_stack: Default::default(),
             commands: Default::default(),
@@ -427,6 +427,9 @@ where
         self.verts.clear();
         self.gradients
             .release_old_gradients(&mut self.images, &mut self.renderer);
+        if let Some(atlas) = self.emphemeral_glyph_atlas.take() {
+            atlas.clear(self);
+        }
     }
 
     pub fn screenshot(&mut self) -> Result<ImgVec<RGBA8>, ErrorKind> {
@@ -1215,15 +1218,16 @@ where
 
         // TODO: Early out if text is outside the canvas bounds, or maybe even check for each character in layout.
 
-        if paint.font_size > 92.0 {
+        let bitmap_glyphs = layout.has_bitmap_glyphs();
+        let need_direct_rendering = paint.font_size > 92.0;
+
+        if need_direct_rendering && !bitmap_glyphs {
             text::render_direct(self, &layout, &paint, render_mode, invscale)?;
         } else {
-            let cmds = text::render_atlas(self, &layout, &paint, render_mode)?;
+            let create_vertices = |quads: &Vec<text::Quad>| {
+                let mut verts = Vec::with_capacity(quads.len() * 6);
 
-            for cmd in &cmds {
-                let mut verts = Vec::with_capacity(cmd.quads.len() * 6);
-
-                for quad in &cmd.quads {
+                for quad in quads {
                     let (p0, p1) = transform.transform_point(quad.x0 * invscale, quad.y0 * invscale);
                     let (p2, p3) = transform.transform_point(quad.x1 * invscale, quad.y0 * invscale);
                     let (p4, p5) = transform.transform_point(quad.x1 * invscale, quad.y1 * invscale);
@@ -1236,8 +1240,34 @@ where
                     verts.push(Vertex::new(p6, p7, quad.s0, quad.t1));
                     verts.push(Vertex::new(p4, p5, quad.s1, quad.t1));
                 }
+                verts
+            };
 
-                paint.set_alpha_mask(Some(cmd.image_id));
+            let atlas = if bitmap_glyphs && need_direct_rendering {
+                self.emphemeral_glyph_atlas
+                    .get_or_insert_with(|| Default::default())
+                    .clone()
+            } else {
+                self.glyph_atlas.clone()
+            };
+
+            let draw_commands = atlas.render_atlas(self, &layout, &paint, render_mode)?;
+
+            for cmd in draw_commands.alpha_glyphs {
+                let verts = create_vertices(&cmd.quads);
+
+                paint.set_glyph_texture(GlyphTexture::AlphaMask(cmd.image_id));
+
+                // Apply global alpha
+                paint.mul_alpha(self.state().alpha);
+
+                self.render_triangles(&verts, &paint);
+            }
+
+            for cmd in draw_commands.color_glyphs {
+                let verts = create_vertices(&cmd.quads);
+
+                paint.set_glyph_texture(GlyphTexture::ColorTexture(cmd.image_id));
 
                 // Apply global alpha
                 paint.mul_alpha(self.state().alpha);
@@ -1258,7 +1288,7 @@ where
 
         let mut cmd = Command::new(CommandType::Triangles { params });
         cmd.composite_operation = self.state().composite_operation;
-        cmd.alpha_mask = paint.alpha_mask();
+        cmd.glyph_texture = paint.glyph_texture();
 
         if let PaintFlavor::Image { id, .. } = paint.flavor {
             cmd.image = Some(id);
@@ -1293,7 +1323,12 @@ where
 
     #[cfg(feature = "debug_inspector")]
     pub fn debug_inspector_get_font_textures(&self) -> Vec<ImageId> {
-        self.glyph_textures.iter().map(|t| t.image_id).collect()
+        self.glyph_atlas
+            .glyph_textures
+            .borrow()
+            .iter()
+            .map(|t| t.image_id)
+            .collect()
     }
 
     #[cfg(feature = "debug_inspector")]
