@@ -247,6 +247,21 @@ impl TextContext {
         self.0.as_ref().borrow_mut().add_font_mem(data)
     }
 
+    /// Registers the in-memory representation of a TrueType font pointed to by the shared data
+    /// parameter with this text context. If successful, the font id is returned. The face_index
+    /// specifies the face index if the font data is a true type font collection. For plain true
+    /// type fonts, use 0 as index.
+    pub fn add_shared_font_with_index<T: AsRef<[u8]> + 'static>(
+        &self,
+        data: T,
+        face_index: u32,
+    ) -> Result<FontId, ErrorKind> {
+        self.0
+            .as_ref()
+            .borrow_mut()
+            .add_shared_font_with_index(data, face_index)
+    }
+
     /// Returns information on how the provided text will be drawn with the specified paint.
     pub fn measure_text<S: AsRef<str>>(&self, x: f32, y: f32, text: S, paint: Paint) -> Result<TextMetrics, ErrorKind> {
         self.0.as_ref().borrow_mut().measure_text(x, y, text, paint)
@@ -309,6 +324,8 @@ impl TextContextImpl {
                 } else {
                     if let Some("ttf") = path.extension().and_then(OsStr::to_str) {
                         fonts.push(self.add_font_file(path)?);
+                    } else if let Some("ttc") = path.extension().and_then(OsStr::to_str) {
+                        fonts.extend(self.add_font_file_collection(path)?);
                     }
                 }
             }
@@ -323,10 +340,35 @@ impl TextContextImpl {
         self.add_font_mem(&data)
     }
 
+    pub fn add_font_file_collection<T: AsRef<FilePath>>(
+        &mut self,
+        path: T,
+    ) -> Result<impl Iterator<Item = FontId> + '_, ErrorKind> {
+        let data = std::fs::read(path)?;
+
+        let count = ttf_parser::fonts_in_collection(&data).unwrap_or(1);
+        Ok((0..count).filter_map(move |index| Some(self.add_font_mem_with_index(&data, index).ok()?)))
+    }
+
     pub fn add_font_mem(&mut self, data: &[u8]) -> Result<FontId, ErrorKind> {
+        self.add_font_mem_with_index(data, 0)
+    }
+
+    pub fn add_font_mem_with_index(&mut self, data: &[u8], face_index: u32) -> Result<FontId, ErrorKind> {
         self.clear_caches();
 
-        let font = Font::new(data)?;
+        let font = Font::new_with_data(data.to_owned(), face_index)?;
+        Ok(FontId(self.fonts.insert(font)))
+    }
+
+    pub fn add_shared_font_with_index<T: AsRef<[u8]> + 'static>(
+        &mut self,
+        data: T,
+        face_index: u32,
+    ) -> Result<FontId, ErrorKind> {
+        self.clear_caches();
+
+        let font = Font::new_with_data(data, face_index)?;
         Ok(FontId(self.fonts.insert(font)))
     }
 
@@ -601,14 +643,7 @@ fn shape_word(
     context.find_font(paint, |(font_id, font)| {
         // Call harfbuzz
         let output = {
-            // TODO: It may be faster if this is created only once and stored inside the Font struct
-            let face = match rustybuzz::Face::from_slice(&font.data(), 0) {
-                Some(v) => v,
-                None => {
-                    eprintln!("Error: malformed font.");
-                    std::process::exit(1);
-                }
-            };
+            let face = font.face_ref();
 
             let mut buffer = rustybuzz::UnicodeBuffer::new();
             buffer.push_str(word);
